# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...types.model import info_list_params
from ..._base_client import make_request_options

__all__ = ["InfoResource", "AsyncInfoResource"]


class InfoResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> InfoResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return InfoResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> InfoResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return InfoResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        litellm_model_id: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Provides more info about each model in /models, including config.yaml
        descriptions (except api key and api base)

        Parameters: litellm_model_id: Optional[str] = None (this is the value of
        `x-litellm-model-id` returned in response headers)

            - When litellm_model_id is passed, it will return the info for that specific model
            - When litellm_model_id is not passed, it will return the info for all models

        Returns: Returns a dictionary containing information about each model.

        Example Response:

        ```json
        {
          "data": [
            {
              "model_name": "fake-openai-endpoint",
              "litellm_params": {
                "api_base": "https://exampleopenaiendpoint-production.up.railway.app/",
                "model": "openai/fake"
              },
              "model_info": {
                "id": "112f74fab24a7a5245d2ced3536dd8f5f9192c57ee6e332af0f0512e08bed5af",
                "db_model": false
              }
            }
          ]
        }
        ```

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/model/info",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"litellm_model_id": litellm_model_id}, info_list_params.InfoListParams),
            ),
            cast_to=object,
        )


class AsyncInfoResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncInfoResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncInfoResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncInfoResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return AsyncInfoResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        litellm_model_id: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Provides more info about each model in /models, including config.yaml
        descriptions (except api key and api base)

        Parameters: litellm_model_id: Optional[str] = None (this is the value of
        `x-litellm-model-id` returned in response headers)

            - When litellm_model_id is passed, it will return the info for that specific model
            - When litellm_model_id is not passed, it will return the info for all models

        Returns: Returns a dictionary containing information about each model.

        Example Response:

        ```json
        {
          "data": [
            {
              "model_name": "fake-openai-endpoint",
              "litellm_params": {
                "api_base": "https://exampleopenaiendpoint-production.up.railway.app/",
                "model": "openai/fake"
              },
              "model_info": {
                "id": "112f74fab24a7a5245d2ced3536dd8f5f9192c57ee6e332af0f0512e08bed5af",
                "db_model": false
              }
            }
          ]
        }
        ```

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/model/info",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"litellm_model_id": litellm_model_id}, info_list_params.InfoListParams
                ),
            ),
            cast_to=object,
        )


class InfoResourceWithRawResponse:
    def __init__(self, info: InfoResource) -> None:
        self._info = info

        self.list = to_raw_response_wrapper(
            info.list,
        )


class AsyncInfoResourceWithRawResponse:
    def __init__(self, info: AsyncInfoResource) -> None:
        self._info = info

        self.list = async_to_raw_response_wrapper(
            info.list,
        )


class InfoResourceWithStreamingResponse:
    def __init__(self, info: InfoResource) -> None:
        self._info = info

        self.list = to_streamed_response_wrapper(
            info.list,
        )


class AsyncInfoResourceWithStreamingResponse:
    def __init__(self, info: AsyncInfoResource) -> None:
        self._info = info

        self.list = async_to_streamed_response_wrapper(
            info.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["BudgetTable"]


class BudgetTable(BaseModel):
    budget_duration: Optional[str] = None

    max_budget: Optional[float] = None

    max_parallel_requests: Optional[int] = None

    api_model_max_budget: Optional[object] = FieldInfo(alias="model_max_budget", default=None)

    rpm_limit: Optional[int] = None

    soft_budget: Optional[float] = None

    tpm_limit: Optional[int] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.provider_list_budgets_response import ProviderListBudgetsResponse

__all__ = ["ProviderResource", "AsyncProviderResource"]


class ProviderResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ProviderResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return ProviderResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ProviderResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return ProviderResourceWithStreamingResponse(self)

    def list_budgets(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ProviderListBudgetsResponse:
        """
        Provider Budget Routing - Get Budget, Spend Details
        https://docs.litellm.ai/docs/proxy/provider_budget_routing

        Use this endpoint to check current budget, spend and budget reset time for a
        provider

        Example Request

        ```bash
        curl -X GET http://localhost:4000/provider/budgets     -H "Content-Type: application/json"     -H "Authorization: Bearer sk-1234"
        ```

        Example Response

        ```json
        {
          "providers": {
            "openai": {
              "budget_limit": 1e-12,
              "time_period": "1d",
              "spend": 0.0,
              "budget_reset_at": null
            },
            "azure": {
              "budget_limit": 100.0,
              "time_period": "1d",
              "spend": 0.0,
              "budget_reset_at": null
            },
            "anthropic": {
              "budget_limit": 100.0,
              "time_period": "10d",
              "spend": 0.0,
              "budget_reset_at": null
            },
            "vertex_ai": {
              "budget_limit": 100.0,
              "time_period": "12d",
              "spend": 0.0,
              "budget_reset_at": null
            }
          }
        }
        ```
        """
        return self._get(
            "/provider/budgets",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProviderListBudgetsResponse,
        )


class AsyncProviderResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncProviderResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncProviderResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncProviderResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return AsyncProviderResourceWithStreamingResponse(self)

    async def list_budgets(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ProviderListBudgetsResponse:
        """
        Provider Budget Routing - Get Budget, Spend Details
        https://docs.litellm.ai/docs/proxy/provider_budget_routing

        Use this endpoint to check current budget, spend and budget reset time for a
        provider

        Example Request

        ```bash
        curl -X GET http://localhost:4000/provider/budgets     -H "Content-Type: application/json"     -H "Authorization: Bearer sk-1234"
        ```

        Example Response

        ```json
        {
          "providers": {
            "openai": {
              "budget_limit": 1e-12,
              "time_period": "1d",
              "spend": 0.0,
              "budget_reset_at": null
            },
            "azure": {
              "budget_limit": 100.0,
              "time_period": "1d",
              "spend": 0.0,
              "budget_reset_at": null
            },
            "anthropic": {
              "budget_limit": 100.0,
              "time_period": "10d",
              "spend": 0.0,
              "budget_reset_at": null
            },
            "vertex_ai": {
              "budget_limit": 100.0,
              "time_period": "12d",
              "spend": 0.0,
              "budget_reset_at": null
            }
          }
        }
        ```
        """
        return await self._get(
            "/provider/budgets",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProviderListBudgetsResponse,
        )


class ProviderResourceWithRawResponse:
    def __init__(self, provider: ProviderResource) -> None:
        self._provider = provider

        self.list_budgets = to_raw_response_wrapper(
            provider.list_budgets,
        )


class AsyncProviderResourceWithRawResponse:
    def __init__(self, provider: AsyncProviderResource) -> None:
        self._provider = provider

        self.list_budgets = async_to_raw_response_wrapper(
            provider.list_budgets,
        )


class ProviderResourceWithStreamingResponse:
    def __init__(self, provider: ProviderResource) -> None:
        self._provider = provider

        self.list_budgets = to_streamed_response_wrapper(
            provider.list_budgets,
        )


class AsyncProviderResourceWithStreamingResponse:
    def __init__(self, provider: AsyncProviderResource) -> None:
        self._provider = provider

        self.list_budgets = async_to_streamed_response_wrapper(
            provider.list_budgets,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["SpendListTagsParams"]


class SpendListTagsParams(TypedDict, total=False):
    end_date: Optional[str]
    """Time till which to view key spend"""

    start_date: Optional[str]
    """Time from which to start viewing key spend"""

    tags: Optional[str]
    """comman separated tags to filter on"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Optional
from typing_extensions import Literal

import httpx

from ..types import (
    customer_block_params,
    customer_create_params,
    customer_delete_params,
    customer_update_params,
    customer_unblock_params,
    customer_retrieve_info_params,
)
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.customer_list_response import CustomerListResponse
from ..types.lite_llm_end_user_table import LiteLlmEndUserTable

__all__ = ["CustomerResource", "AsyncCustomerResource"]


class CustomerResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CustomerResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return CustomerResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CustomerResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return CustomerResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        user_id: str,
        alias: Optional[str] | NotGiven = NOT_GIVEN,
        allowed_model_region: Optional[Literal["eu", "us"]] | NotGiven = NOT_GIVEN,
        blocked: bool | NotGiven = NOT_GIVEN,
        budget_duration: Optional[str] | NotGiven = NOT_GIVEN,
        budget_id: Optional[str] | NotGiven = NOT_GIVEN,
        default_model: Optional[str] | NotGiven = NOT_GIVEN,
        max_budget: Optional[float] | NotGiven = NOT_GIVEN,
        max_parallel_requests: Optional[int] | NotGiven = NOT_GIVEN,
        model_max_budget: Optional[Dict[str, customer_create_params.ModelMaxBudget]] | NotGiven = NOT_GIVEN,
        rpm_limit: Optional[int] | NotGiven = NOT_GIVEN,
        soft_budget: Optional[float] | NotGiven = NOT_GIVEN,
        tpm_limit: Optional[int] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Allow creating a new Customer

        Parameters:

        - user_id: str - The unique identifier for the user.
        - alias: Optional[str] - A human-friendly alias for the user.
        - blocked: bool - Flag to allow or disallow requests for this end-user. Default
          is False.
        - max_budget: Optional[float] - The maximum budget allocated to the user. Either
          'max_budget' or 'budget_id' should be provided, not both.
        - budget_id: Optional[str] - The identifier for an existing budget allocated to
          the user. Either 'max_budget' or 'budget_id' should be provided, not both.
        - allowed_model_region: Optional[Union[Literal["eu"], Literal["us"]]] - Require
          all user requests to use models in this specific region.
        - default_model: Optional[str] - If no equivalent model in the allowed region,
          default all requests to this model.
        - metadata: Optional[dict] = Metadata for customer, store information for
          customer. Example metadata = {"data_training_opt_out": True}
        - budget_duration: Optional[str] - Budget is reset at the end of specified
          duration. If not set, budget is never reset. You can set duration as seconds
          ("30s"), minutes ("30m"), hours ("30h"), days ("30d").
        - tpm_limit: Optional[int] - [Not Implemented Yet] Specify tpm limit for a given
          customer (Tokens per minute)
        - rpm_limit: Optional[int] - [Not Implemented Yet] Specify rpm limit for a given
          customer (Requests per minute)
        - model_max_budget: Optional[dict] - [Not Implemented Yet] Specify max budget
          for a given model. Example: {"openai/gpt-4o-mini": {"max_budget": 100.0,
          "budget_duration": "1d"}}
        - max_parallel_requests: Optional[int] - [Not Implemented Yet] Specify max
          parallel requests for a given customer.
        - soft_budget: Optional[float] - [Not Implemented Yet] Get alerts when customer
          crosses given budget, doesn't block requests.

        - Allow specifying allowed regions
        - Allow specifying default model

        Example curl:

        ```
        curl --location 'http://0.0.0.0:4000/customer/new'         --header 'Authorization: Bearer sk-1234'         --header 'Content-Type: application/json'         --data '{
                "user_id" : "ishaan-jaff-3",
                "allowed_region": "eu",
                "budget_id": "free_tier",
                "default_model": "azure/gpt-3.5-turbo-eu" <- all calls from this user, use this model?
            }'

            # return end-user object
        ```

        NOTE: This used to be called `/end_user/new`, we will still be maintaining
        compatibility for /end_user/XXX for these endpoints

        Args:
          budget_duration: Max duration budget should be set for (e.g. '1hr', '1d', '28d')

          max_budget: Requests will fail if this budget (in USD) is exceeded.

          max_parallel_requests: Max concurrent requests allowed for this budget id.

          model_max_budget: Max budget for each model (e.g. {'gpt-4o': {'max_budget': '0.0000001',
              'budget_duration': '1d', 'tpm_limit': 1000, 'rpm_limit': 1000}})

          rpm_limit: Max requests per minute, allowed for this budget id.

          soft_budget: Requests will NOT fail if this is exceeded. Will fire alerting though.

          tpm_limit: Max tokens per minute, allowed for this budget id.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/customer/new",
            body=maybe_transform(
                {
                    "user_id": user_id,
                    "alias": alias,
                    "allowed_model_region": allowed_model_region,
                    "blocked": blocked,
                    "budget_duration": budget_duration,
                    "budget_id": budget_id,
                    "default_model": default_model,
                    "max_budget": max_budget,
                    "max_parallel_requests": max_parallel_requests,
                    "model_max_budget": model_max_budget,
                    "rpm_limit": rpm_limit,
                    "soft_budget": soft_budget,
                    "tpm_limit": tpm_limit,
                },
                customer_create_params.CustomerCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def update(
        self,
        *,
        user_id: str,
        alias: Optional[str] | NotGiven = NOT_GIVEN,
        allowed_model_region: Optional[Literal["eu", "us"]] | NotGiven = NOT_GIVEN,
        blocked: bool | NotGiven = NOT_GIVEN,
        budget_id: Optional[str] | NotGiven = NOT_GIVEN,
        default_model: Optional[str] | NotGiven = NOT_GIVEN,
        max_budget: Optional[float] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Example curl

        Parameters:

        - user_id: str
        - alias: Optional[str] = None # human-friendly alias
        - blocked: bool = False # allow/disallow requests for this end-user
        - max_budget: Optional[float] = None
        - budget_id: Optional[str] = None # give either a budget_id or max_budget
        - allowed_model_region: Optional[AllowedModelRegion] = ( None # require all user
          requests to use models in this specific region )
        - default_model: Optional[str] = ( None # if no equivalent model in allowed
          region - default all requests to this model )

        Example curl:

        ```
        curl --location 'http://0.0.0.0:4000/customer/update'     --header 'Authorization: Bearer sk-1234'     --header 'Content-Type: application/json'     --data '{
            "user_id": "test-litellm-user-4",
            "budget_id": "paid_tier"
        }'

        See below for all params
        ```

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/customer/update",
            body=maybe_transform(
                {
                    "user_id": user_id,
                    "alias": alias,
                    "allowed_model_region": allowed_model_region,
                    "blocked": blocked,
                    "budget_id": budget_id,
                    "default_model": default_model,
                    "max_budget": max_budget,
                },
                customer_update_params.CustomerUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CustomerListResponse:
        """
        [Admin-only] List all available customers

        Example curl:

        ```
        curl --location --request GET 'http://0.0.0.0:4000/customer/list'         --header 'Authorization: Bearer sk-1234'
        ```
        """
        return self._get(
            "/customer/list",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerListResponse,
        )

    def delete(
        self,
        *,
        user_ids: List[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Delete multiple end-users.

        Parameters:

        - user_ids (List[str], required): The unique `user_id`s for the users to delete

        Example curl:

        ```
        curl --location 'http://0.0.0.0:4000/customer/delete'         --header 'Authorization: Bearer sk-1234'         --header 'Content-Type: application/json'         --data '{
                "user_ids" :["ishaan-jaff-5"]
        }'

        See below for all params
        ```

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/customer/delete",
            body=maybe_transform({"user_ids": user_ids}, customer_delete_params.CustomerDeleteParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def block(
        self,
        *,
        user_ids: List[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        [BETA] Reject calls with this end-user id

        Parameters:

        - user_ids (List[str], required): The unique `user_id`s for the users to block

          (any /chat/completion call with this user={end-user-id} param, will be
          rejected.)

          ```
          curl -X POST "http://0.0.0.0:8000/user/block"
          -H "Authorization: Bearer sk-1234"
          -d '{
          "user_ids": [<user_id>, ...]
          }'
          ```

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/customer/block",
            body=maybe_transform({"user_ids": user_ids}, customer_block_params.CustomerBlockParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def retrieve_info(
        self,
        *,
        end_user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> LiteLlmEndUserTable:
        """Get information about an end-user.

        An `end_user` is a customer (external user)
        of the proxy.

        Parameters:

        - end_user_id (str, required): The unique identifier for the end-user

        Example curl:

        ```
        curl -X GET 'http://localhost:4000/customer/info?end_user_id=test-litellm-user-4'         -H 'Authorization: Bearer sk-1234'
        ```

        Args:
          end_user_id: End User ID in the request parameters

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/customer/info",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"end_user_id": end_user_id}, customer_retrieve_info_params.CustomerRetrieveInfoParams
                ),
            ),
            cast_to=LiteLlmEndUserTable,
        )

    def unblock(
        self,
        *,
        user_ids: List[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        [BETA] Unblock calls with this user id

        Example

        ```
        curl -X POST "http://0.0.0.0:8000/user/unblock"
        -H "Authorization: Bearer sk-1234"
        -d '{
        "user_ids": [<user_id>, ...]
        }'
        ```

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/customer/unblock",
            body=maybe_transform({"user_ids": user_ids}, customer_unblock_params.CustomerUnblockParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AsyncCustomerResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCustomerResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncCustomerResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCustomerResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return AsyncCustomerResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        user_id: str,
        alias: Optional[str] | NotGiven = NOT_GIVEN,
        allowed_model_region: Optional[Literal["eu", "us"]] | NotGiven = NOT_GIVEN,
        blocked: bool | NotGiven = NOT_GIVEN,
        budget_duration: Optional[str] | NotGiven = NOT_GIVEN,
        budget_id: Optional[str] | NotGiven = NOT_GIVEN,
        default_model: Optional[str] | NotGiven = NOT_GIVEN,
        max_budget: Optional[float] | NotGiven = NOT_GIVEN,
        max_parallel_requests: Optional[int] | NotGiven = NOT_GIVEN,
        model_max_budget: Optional[Dict[str, customer_create_params.ModelMaxBudget]] | NotGiven = NOT_GIVEN,
        rpm_limit: Optional[int] | NotGiven = NOT_GIVEN,
        soft_budget: Optional[float] | NotGiven = NOT_GIVEN,
        tpm_limit: Optional[int] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Allow creating a new Customer

        Parameters:

        - user_id: str - The unique identifier for the user.
        - alias: Optional[str] - A human-friendly alias for the user.
        - blocked: bool - Flag to allow or disallow requests for this end-user. Default
          is False.
        - max_budget: Optional[float] - The maximum budget allocated to the user. Either
          'max_budget' or 'budget_id' should be provided, not both.
        - budget_id: Optional[str] - The identifier for an existing budget allocated to
          the user. Either 'max_budget' or 'budget_id' should be provided, not both.
        - allowed_model_region: Optional[Union[Literal["eu"], Literal["us"]]] - Require
          all user requests to use models in this specific region.
        - default_model: Optional[str] - If no equivalent model in the allowed region,
          default all requests to this model.
        - metadata: Optional[dict] = Metadata for customer, store information for
          customer. Example metadata = {"data_training_opt_out": True}
        - budget_duration: Optional[str] - Budget is reset at the end of specified
          duration. If not set, budget is never reset. You can set duration as seconds
          ("30s"), minutes ("30m"), hours ("30h"), days ("30d").
        - tpm_limit: Optional[int] - [Not Implemented Yet] Specify tpm limit for a given
          customer (Tokens per minute)
        - rpm_limit: Optional[int] - [Not Implemented Yet] Specify rpm limit for a given
          customer (Requests per minute)
        - model_max_budget: Optional[dict] - [Not Implemented Yet] Specify max budget
          for a given model. Example: {"openai/gpt-4o-mini": {"max_budget": 100.0,
          "budget_duration": "1d"}}
        - max_parallel_requests: Optional[int] - [Not Implemented Yet] Specify max
          parallel requests for a given customer.
        - soft_budget: Optional[float] - [Not Implemented Yet] Get alerts when customer
          crosses given budget, doesn't block requests.

        - Allow specifying allowed regions
        - Allow specifying default model

        Example curl:

        ```
        curl --location 'http://0.0.0.0:4000/customer/new'         --header 'Authorization: Bearer sk-1234'         --header 'Content-Type: application/json'         --data '{
                "user_id" : "ishaan-jaff-3",
                "allowed_region": "eu",
                "budget_id": "free_tier",
                "default_model": "azure/gpt-3.5-turbo-eu" <- all calls from this user, use this model?
            }'

            # return end-user object
        ```

        NOTE: This used to be called `/end_user/new`, we will still be maintaining
        compatibility for /end_user/XXX for these endpoints

        Args:
          budget_duration: Max duration budget should be set for (e.g. '1hr', '1d', '28d')

          max_budget: Requests will fail if this budget (in USD) is exceeded.

          max_parallel_requests: Max concurrent requests allowed for this budget id.

          model_max_budget: Max budget for each model (e.g. {'gpt-4o': {'max_budget': '0.0000001',
              'budget_duration': '1d', 'tpm_limit': 1000, 'rpm_limit': 1000}})

          rpm_limit: Max requests per minute, allowed for this budget id.

          soft_budget: Requests will NOT fail if this is exceeded. Will fire alerting though.

          tpm_limit: Max tokens per minute, allowed for this budget id.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/customer/new",
            body=await async_maybe_transform(
                {
                    "user_id": user_id,
                    "alias": alias,
                    "allowed_model_region": allowed_model_region,
                    "blocked": blocked,
                    "budget_duration": budget_duration,
                    "budget_id": budget_id,
                    "default_model": default_model,
                    "max_budget": max_budget,
                    "max_parallel_requests": max_parallel_requests,
                    "model_max_budget": model_max_budget,
                    "rpm_limit": rpm_limit,
                    "soft_budget": soft_budget,
                    "tpm_limit": tpm_limit,
                },
                customer_create_params.CustomerCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def update(
        self,
        *,
        user_id: str,
        alias: Optional[str] | NotGiven = NOT_GIVEN,
        allowed_model_region: Optional[Literal["eu", "us"]] | NotGiven = NOT_GIVEN,
        blocked: bool | NotGiven = NOT_GIVEN,
        budget_id: Optional[str] | NotGiven = NOT_GIVEN,
        default_model: Optional[str] | NotGiven = NOT_GIVEN,
        max_budget: Optional[float] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Example curl

        Parameters:

        - user_id: str
        - alias: Optional[str] = None # human-friendly alias
        - blocked: bool = False # allow/disallow requests for this end-user
        - max_budget: Optional[float] = None
        - budget_id: Optional[str] = None # give either a budget_id or max_budget
        - allowed_model_region: Optional[AllowedModelRegion] = ( None # require all user
          requests to use models in this specific region )
        - default_model: Optional[str] = ( None # if no equivalent model in allowed
          region - default all requests to this model )

        Example curl:

        ```
        curl --location 'http://0.0.0.0:4000/customer/update'     --header 'Authorization: Bearer sk-1234'     --header 'Content-Type: application/json'     --data '{
            "user_id": "test-litellm-user-4",
            "budget_id": "paid_tier"
        }'

        See below for all params
        ```

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/customer/update",
            body=await async_maybe_transform(
                {
                    "user_id": user_id,
                    "alias": alias,
                    "allowed_model_region": allowed_model_region,
                    "blocked": blocked,
                    "budget_id": budget_id,
                    "default_model": default_model,
                    "max_budget": max_budget,
                },
                customer_update_params.CustomerUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CustomerListResponse:
        """
        [Admin-only] List all available customers

        Example curl:

        ```
        curl --location --request GET 'http://0.0.0.0:4000/customer/list'         --header 'Authorization: Bearer sk-1234'
        ```
        """
        return await self._get(
            "/customer/list",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerListResponse,
        )

    async def delete(
        self,
        *,
        user_ids: List[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Delete multiple end-users.

        Parameters:

        - user_ids (List[str], required): The unique `user_id`s for the users to delete

        Example curl:

        ```
        curl --location 'http://0.0.0.0:4000/customer/delete'         --header 'Authorization: Bearer sk-1234'         --header 'Content-Type: application/json'         --data '{
                "user_ids" :["ishaan-jaff-5"]
        }'

        See below for all params
        ```

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/customer/delete",
            body=await async_maybe_transform({"user_ids": user_ids}, customer_delete_params.CustomerDeleteParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def block(
        self,
        *,
        user_ids: List[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        [BETA] Reject calls with this end-user id

        Parameters:

        - user_ids (List[str], required): The unique `user_id`s for the users to block

          (any /chat/completion call with this user={end-user-id} param, will be
          rejected.)

          ```
          curl -X POST "http://0.0.0.0:8000/user/block"
          -H "Authorization: Bearer sk-1234"
          -d '{
          "user_ids": [<user_id>, ...]
          }'
          ```

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/customer/block",
            body=await async_maybe_transform({"user_ids": user_ids}, customer_block_params.CustomerBlockParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def retrieve_info(
        self,
        *,
        end_user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> LiteLlmEndUserTable:
        """Get information about an end-user.

        An `end_user` is a customer (external user)
        of the proxy.

        Parameters:

        - end_user_id (str, required): The unique identifier for the end-user

        Example curl:

        ```
        curl -X GET 'http://localhost:4000/customer/info?end_user_id=test-litellm-user-4'         -H 'Authorization: Bearer sk-1234'
        ```

        Args:
          end_user_id: End User ID in the request parameters

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/customer/info",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"end_user_id": end_user_id}, customer_retrieve_info_params.CustomerRetrieveInfoParams
                ),
            ),
            cast_to=LiteLlmEndUserTable,
        )

    async def unblock(
        self,
        *,
        user_ids: List[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        [BETA] Unblock calls with this user id

        Example

        ```
        curl -X POST "http://0.0.0.0:8000/user/unblock"
        -H "Authorization: Bearer sk-1234"
        -d '{
        "user_ids": [<user_id>, ...]
        }'
        ```

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/customer/unblock",
            body=await async_maybe_transform({"user_ids": user_ids}, customer_unblock_params.CustomerUnblockParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class CustomerResourceWithRawResponse:
    def __init__(self, customer: CustomerResource) -> None:
        self._customer = customer

        self.create = to_raw_response_wrapper(
            customer.create,
        )
        self.update = to_raw_response_wrapper(
            customer.update,
        )
        self.list = to_raw_response_wrapper(
            customer.list,
        )
        self.delete = to_raw_response_wrapper(
            customer.delete,
        )
        self.block = to_raw_response_wrapper(
            customer.block,
        )
        self.retrieve_info = to_raw_response_wrapper(
            customer.retrieve_info,
        )
        self.unblock = to_raw_response_wrapper(
            customer.unblock,
        )


class AsyncCustomerResourceWithRawResponse:
    def __init__(self, customer: AsyncCustomerResource) -> None:
        self._customer = customer

        self.create = async_to_raw_response_wrapper(
            customer.create,
        )
        self.update = async_to_raw_response_wrapper(
            customer.update,
        )
        self.list = async_to_raw_response_wrapper(
            customer.list,
        )
        self.delete = async_to_raw_response_wrapper(
            customer.delete,
        )
        self.block = async_to_raw_response_wrapper(
            customer.block,
        )
        self.retrieve_info = async_to_raw_response_wrapper(
            customer.retrieve_info,
        )
        self.unblock = async_to_raw_response_wrapper(
            customer.unblock,
        )


class CustomerResourceWithStreamingResponse:
    def __init__(self, customer: CustomerResource) -> None:
        self._customer = customer

        self.create = to_streamed_response_wrapper(
            customer.create,
        )
        self.update = to_streamed_response_wrapper(
            customer.update,
        )
        self.list = to_streamed_response_wrapper(
            customer.list,
        )
        self.delete = to_streamed_response_wrapper(
            customer.delete,
        )
        self.block = to_streamed_response_wrapper(
            customer.block,
        )
        self.retrieve_info = to_streamed_response_wrapper(
            customer.retrieve_info,
        )
        self.unblock = to_streamed_response_wrapper(
            customer.unblock,
        )


class AsyncCustomerResourceWithStreamingResponse:
    def __init__(self, customer: AsyncCustomerResource) -> None:
        self._customer = customer

        self.create = async_to_streamed_response_wrapper(
            customer.create,
        )
        self.update = async_to_streamed_response_wrapper(
            customer.update,
        )
        self.list = async_to_streamed_response_wrapper(
            customer.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            customer.delete,
        )
        self.block = async_to_streamed_response_wrapper(
            customer.block,
        )
        self.retrieve_info = async_to_streamed_response_wrapper(
            customer.retrieve_info,
        )
        self.unblock = async_to_streamed_response_wrapper(
            customer.unblock,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options

__all__ = ["SpeechResource", "AsyncSpeechResource"]


class SpeechResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SpeechResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return SpeechResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SpeechResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return SpeechResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Same params as:

        https://platform.openai.com/docs/api-reference/audio/createSpeech
        """
        return self._post(
            "/v1/audio/speech",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AsyncSpeechResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSpeechResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncSpeechResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSpeechResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return AsyncSpeechResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Same params as:

        https://platform.openai.com/docs/api-reference/audio/createSpeech
        """
        return await self._post(
            "/v1/audio/speech",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class SpeechResourceWithRawResponse:
    def __init__(self, speech: SpeechResource) -> None:
        self._speech = speech

        self.create = to_raw_response_wrapper(
            speech.create,
        )


class AsyncSpeechResourceWithRawResponse:
    def __init__(self, speech: AsyncSpeechResource) -> None:
        self._speech = speech

        self.create = async_to_raw_response_wrapper(
            speech.create,
        )


class SpeechResourceWithStreamingResponse:
    def __init__(self, speech: SpeechResource) -> None:
        self._speech = speech

        self.create = to_streamed_response_wrapper(
            speech.create,
        )


class AsyncSpeechResourceWithStreamingResponse:
    def __init__(self, speech: AsyncSpeechResource) -> None:
        self._speech = speech

        self.create = async_to_streamed_response_wrapper(
            speech.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable, Optional
from typing_extensions import Required, TypeAlias, TypedDict

from .org_member_param import OrgMemberParam

__all__ = ["OrganizationAddMemberParams", "Member"]


class OrganizationAddMemberParams(TypedDict, total=False):
    member: Required[Member]

    organization_id: Required[str]

    max_budget_in_organization: Optional[float]


Member: TypeAlias = Union[Iterable[OrgMemberParam], OrgMemberParam]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options

__all__ = ["ModerationsResource", "AsyncModerationsResource"]


class ModerationsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ModerationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return ModerationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ModerationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return ModerationsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        The moderations endpoint is a tool you can use to check whether content complies
        with an LLM Providers policies.

        Quick Start

        ```
        curl --location 'http://0.0.0.0:4000/moderations'     --header 'Content-Type: application/json'     --header 'Authorization: Bearer sk-1234'     --data '{"input": "Sample text goes here", "model": "text-moderation-stable"}'
        ```
        """
        return self._post(
            "/v1/moderations",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AsyncModerationsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncModerationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncModerationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncModerationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return AsyncModerationsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        The moderations endpoint is a tool you can use to check whether content complies
        with an LLM Providers policies.

        Quick Start

        ```
        curl --location 'http://0.0.0.0:4000/moderations'     --header 'Content-Type: application/json'     --header 'Authorization: Bearer sk-1234'     --data '{"input": "Sample text goes here", "model": "text-moderation-stable"}'
        ```
        """
        return await self._post(
            "/v1/moderations",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class ModerationsResourceWithRawResponse:
    def __init__(self, moderations: ModerationsResource) -> None:
        self._moderations = moderations

        self.create = to_raw_response_wrapper(
            moderations.create,
        )


class AsyncModerationsResourceWithRawResponse:
    def __init__(self, moderations: AsyncModerationsResource) -> None:
        self._moderations = moderations

        self.create = async_to_raw_response_wrapper(
            moderations.create,
        )


class ModerationsResourceWithStreamingResponse:
    def __init__(self, moderations: ModerationsResource) -> None:
        self._moderations = moderations

        self.create = to_streamed_response_wrapper(
            moderations.create,
        )


class AsyncModerationsResourceWithStreamingResponse:
    def __init__(self, moderations: AsyncModerationsResource) -> None:
        self._moderations = moderations

        self.create = async_to_streamed_response_wrapper(
            moderations.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from .cancel import (
    CancelResource,
    AsyncCancelResource,
    CancelResourceWithRawResponse,
    AsyncCancelResourceWithRawResponse,
    CancelResourceWithStreamingResponse,
    AsyncCancelResourceWithStreamingResponse,
)
from ...types import (
    batch_list_params,
    batch_create_params,
    batch_retrieve_params,
    batch_list_with_provider_params,
)
from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options

__all__ = ["BatchesResource", "AsyncBatchesResource"]


class BatchesResource(SyncAPIResource):
    @cached_property
    def cancel(self) -> CancelResource:
        return CancelResource(self._client)

    @cached_property
    def with_raw_response(self) -> BatchesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return BatchesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BatchesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return BatchesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        provider: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """Create large batches of API requests for asynchronous processing.

        This is the
        equivalent of POST https://api.openai.com/v1/batch Supports Identical Params as:
        https://platform.openai.com/docs/api-reference/batch

        Example Curl

        ```
        curl http://localhost:4000/v1/batches         -H "Authorization: Bearer sk-1234"         -H "Content-Type: application/json"         -d '{
                "input_file_id": "file-abc123",
                "endpoint": "/v1/chat/completions",
                "completion_window": "24h"
        }'
        ```

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/batches",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"provider": provider}, batch_create_params.BatchCreateParams),
            ),
            cast_to=object,
        )

    def retrieve(
        self,
        batch_id: str,
        *,
        provider: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """Retrieves a batch.

        This is the equivalent of GET
        https://api.openai.com/v1/batches/{batch_id} Supports Identical Params as:
        https://platform.openai.com/docs/api-reference/batch/retrieve

        Example Curl

        ```
        curl http://localhost:4000/v1/batches/batch_abc123     -H "Authorization: Bearer sk-1234"     -H "Content-Type: application/json"
        ```

        Args:
          batch_id: The ID of the batch to retrieve

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not batch_id:
            raise ValueError(f"Expected a non-empty value for `batch_id` but received {batch_id!r}")
        return self._get(
            f"/v1/batches/{batch_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"provider": provider}, batch_retrieve_params.BatchRetrieveParams),
            ),
            cast_to=object,
        )

    def list(
        self,
        *,
        after: Optional[str] | NotGiven = NOT_GIVEN,
        limit: Optional[int] | NotGiven = NOT_GIVEN,
        provider: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Lists This is the equivalent of GET https://api.openai.com/v1/batches/ Supports
        Identical Params as: https://platform.openai.com/docs/api-reference/batch/list

        Example Curl

        ```
        curl http://localhost:4000/v1/batches?limit=2     -H "Authorization: Bearer sk-1234"     -H "Content-Type: application/json"
        ```

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/v1/batches",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "limit": limit,
                        "provider": provider,
                    },
                    batch_list_params.BatchListParams,
                ),
            ),
            cast_to=object,
        )

    def cancel_with_provider(
        self,
        batch_id: str,
        *,
        provider: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """Cancel a batch.

        This is the equivalent of POST
        https://api.openai.com/v1/batches/{batch_id}/cancel

        Supports Identical Params as:
        https://platform.openai.com/docs/api-reference/batch/cancel

        Example Curl

        ```
        curl http://localhost:4000/v1/batches/batch_abc123/cancel         -H "Authorization: Bearer sk-1234"         -H "Content-Type: application/json"         -X POST

        ```

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not provider:
            raise ValueError(f"Expected a non-empty value for `provider` but received {provider!r}")
        if not batch_id:
            raise ValueError(f"Expected a non-empty value for `batch_id` but received {batch_id!r}")
        return self._post(
            f"/{provider}/v1/batches/{batch_id}/cancel",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def create_with_provider(
        self,
        provider: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """Create large batches of API requests for asynchronous processing.

        This is the
        equivalent of POST https://api.openai.com/v1/batch Supports Identical Params as:
        https://platform.openai.com/docs/api-reference/batch

        Example Curl

        ```
        curl http://localhost:4000/v1/batches         -H "Authorization: Bearer sk-1234"         -H "Content-Type: application/json"         -d '{
                "input_file_id": "file-abc123",
                "endpoint": "/v1/chat/completions",
                "completion_window": "24h"
        }'
        ```

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not provider:
            raise ValueError(f"Expected a non-empty value for `provider` but received {provider!r}")
        return self._post(
            f"/{provider}/v1/batches",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def list_with_provider(
        self,
        provider: str,
        *,
        after: Optional[str] | NotGiven = NOT_GIVEN,
        limit: Optional[int] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Lists This is the equivalent of GET https://api.openai.com/v1/batches/ Supports
        Identical Params as: https://platform.openai.com/docs/api-reference/batch/list

        Example Curl

        ```
        curl http://localhost:4000/v1/batches?limit=2     -H "Authorization: Bearer sk-1234"     -H "Content-Type: application/json"
        ```

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not provider:
            raise ValueError(f"Expected a non-empty value for `provider` but received {provider!r}")
        return self._get(
            f"/{provider}/v1/batches",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "limit": limit,
                    },
                    batch_list_with_provider_params.BatchListWithProviderParams,
                ),
            ),
            cast_to=object,
        )

    def retrieve_with_provider(
        self,
        batch_id: str,
        *,
        provider: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """Retrieves a batch.

        This is the equivalent of GET
        https://api.openai.com/v1/batches/{batch_id} Supports Identical Params as:
        https://platform.openai.com/docs/api-reference/batch/retrieve

        Example Curl

        ```
        curl http://localhost:4000/v1/batches/batch_abc123     -H "Authorization: Bearer sk-1234"     -H "Content-Type: application/json"
        ```

        Args:
          batch_id: The ID of the batch to retrieve

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not provider:
            raise ValueError(f"Expected a non-empty value for `provider` but received {provider!r}")
        if not batch_id:
            raise ValueError(f"Expected a non-empty value for `batch_id` but received {batch_id!r}")
        return self._get(
            f"/{provider}/v1/batches/{batch_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AsyncBatchesResource(AsyncAPIResource):
    @cached_property
    def cancel(self) -> AsyncCancelResource:
        return AsyncCancelResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncBatchesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncBatchesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBatchesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return AsyncBatchesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        provider: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """Create large batches of API requests for asynchronous processing.

        This is the
        equivalent of POST https://api.openai.com/v1/batch Supports Identical Params as:
        https://platform.openai.com/docs/api-reference/batch

        Example Curl

        ```
        curl http://localhost:4000/v1/batches         -H "Authorization: Bearer sk-1234"         -H "Content-Type: application/json"         -d '{
                "input_file_id": "file-abc123",
                "endpoint": "/v1/chat/completions",
                "completion_window": "24h"
        }'
        ```

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/batches",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"provider": provider}, batch_create_params.BatchCreateParams),
            ),
            cast_to=object,
        )

    async def retrieve(
        self,
        batch_id: str,
        *,
        provider: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """Retrieves a batch.

        This is the equivalent of GET
        https://api.openai.com/v1/batches/{batch_id} Supports Identical Params as:
        https://platform.openai.com/docs/api-reference/batch/retrieve

        Example Curl

        ```
        curl http://localhost:4000/v1/batches/batch_abc123     -H "Authorization: Bearer sk-1234"     -H "Content-Type: application/json"
        ```

        Args:
          batch_id: The ID of the batch to retrieve

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not batch_id:
            raise ValueError(f"Expected a non-empty value for `batch_id` but received {batch_id!r}")
        return await self._get(
            f"/v1/batches/{batch_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"provider": provider}, batch_retrieve_params.BatchRetrieveParams),
            ),
            cast_to=object,
        )

    async def list(
        self,
        *,
        after: Optional[str] | NotGiven = NOT_GIVEN,
        limit: Optional[int] | NotGiven = NOT_GIVEN,
        provider: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Lists This is the equivalent of GET https://api.openai.com/v1/batches/ Supports
        Identical Params as: https://platform.openai.com/docs/api-reference/batch/list

        Example Curl

        ```
        curl http://localhost:4000/v1/batches?limit=2     -H "Authorization: Bearer sk-1234"     -H "Content-Type: application/json"
        ```

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/v1/batches",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "after": after,
                        "limit": limit,
                        "provider": provider,
                    },
                    batch_list_params.BatchListParams,
                ),
            ),
            cast_to=object,
        )

    async def cancel_with_provider(
        self,
        batch_id: str,
        *,
        provider: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """Cancel a batch.

        This is the equivalent of POST
        https://api.openai.com/v1/batches/{batch_id}/cancel

        Supports Identical Params as:
        https://platform.openai.com/docs/api-reference/batch/cancel

        Example Curl

        ```
        curl http://localhost:4000/v1/batches/batch_abc123/cancel         -H "Authorization: Bearer sk-1234"         -H "Content-Type: application/json"         -X POST

        ```

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not provider:
            raise ValueError(f"Expected a non-empty value for `provider` but received {provider!r}")
        if not batch_id:
            raise ValueError(f"Expected a non-empty value for `batch_id` but received {batch_id!r}")
        return await self._post(
            f"/{provider}/v1/batches/{batch_id}/cancel",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def create_with_provider(
        self,
        provider: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """Create large batches of API requests for asynchronous processing.

        This is the
        equivalent of POST https://api.openai.com/v1/batch Supports Identical Params as:
        https://platform.openai.com/docs/api-reference/batch

        Example Curl

        ```
        curl http://localhost:4000/v1/batches         -H "Authorization: Bearer sk-1234"         -H "Content-Type: application/json"         -d '{
                "input_file_id": "file-abc123",
                "endpoint": "/v1/chat/completions",
                "completion_window": "24h"
        }'
        ```

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not provider:
            raise ValueError(f"Expected a non-empty value for `provider` but received {provider!r}")
        return await self._post(
            f"/{provider}/v1/batches",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def list_with_provider(
        self,
        provider: str,
        *,
        after: Optional[str] | NotGiven = NOT_GIVEN,
        limit: Optional[int] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Lists This is the equivalent of GET https://api.openai.com/v1/batches/ Supports
        Identical Params as: https://platform.openai.com/docs/api-reference/batch/list

        Example Curl

        ```
        curl http://localhost:4000/v1/batches?limit=2     -H "Authorization: Bearer sk-1234"     -H "Content-Type: application/json"
        ```

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not provider:
            raise ValueError(f"Expected a non-empty value for `provider` but received {provider!r}")
        return await self._get(
            f"/{provider}/v1/batches",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "after": after,
                        "limit": limit,
                    },
                    batch_list_with_provider_params.BatchListWithProviderParams,
                ),
            ),
            cast_to=object,
        )

    async def retrieve_with_provider(
        self,
        batch_id: str,
        *,
        provider: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """Retrieves a batch.

        This is the equivalent of GET
        https://api.openai.com/v1/batches/{batch_id} Supports Identical Params as:
        https://platform.openai.com/docs/api-reference/batch/retrieve

        Example Curl

        ```
        curl http://localhost:4000/v1/batches/batch_abc123     -H "Authorization: Bearer sk-1234"     -H "Content-Type: application/json"
        ```

        Args:
          batch_id: The ID of the batch to retrieve

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not provider:
            raise ValueError(f"Expected a non-empty value for `provider` but received {provider!r}")
        if not batch_id:
            raise ValueError(f"Expected a non-empty value for `batch_id` but received {batch_id!r}")
        return await self._get(
            f"/{provider}/v1/batches/{batch_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class BatchesResourceWithRawResponse:
    def __init__(self, batches: BatchesResource) -> None:
        self._batches = batches

        self.create = to_raw_response_wrapper(
            batches.create,
        )
        self.retrieve = to_raw_response_wrapper(
            batches.retrieve,
        )
        self.list = to_raw_response_wrapper(
            batches.list,
        )
        self.cancel_with_provider = to_raw_response_wrapper(
            batches.cancel_with_provider,
        )
        self.create_with_provider = to_raw_response_wrapper(
            batches.create_with_provider,
        )
        self.list_with_provider = to_raw_response_wrapper(
            batches.list_with_provider,
        )
        self.retrieve_with_provider = to_raw_response_wrapper(
            batches.retrieve_with_provider,
        )

    @cached_property
    def cancel(self) -> CancelResourceWithRawResponse:
        return CancelResourceWithRawResponse(self._batches.cancel)


class AsyncBatchesResourceWithRawResponse:
    def __init__(self, batches: AsyncBatchesResource) -> None:
        self._batches = batches

        self.create = async_to_raw_response_wrapper(
            batches.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            batches.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            batches.list,
        )
        self.cancel_with_provider = async_to_raw_response_wrapper(
            batches.cancel_with_provider,
        )
        self.create_with_provider = async_to_raw_response_wrapper(
            batches.create_with_provider,
        )
        self.list_with_provider = async_to_raw_response_wrapper(
            batches.list_with_provider,
        )
        self.retrieve_with_provider = async_to_raw_response_wrapper(
            batches.retrieve_with_provider,
        )

    @cached_property
    def cancel(self) -> AsyncCancelResourceWithRawResponse:
        return AsyncCancelResourceWithRawResponse(self._batches.cancel)


class BatchesResourceWithStreamingResponse:
    def __init__(self, batches: BatchesResource) -> None:
        self._batches = batches

        self.create = to_streamed_response_wrapper(
            batches.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            batches.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            batches.list,
        )
        self.cancel_with_provider = to_streamed_response_wrapper(
            batches.cancel_with_provider,
        )
        self.create_with_provider = to_streamed_response_wrapper(
            batches.create_with_provider,
        )
        self.list_with_provider = to_streamed_response_wrapper(
            batches.list_with_provider,
        )
        self.retrieve_with_provider = to_streamed_response_wrapper(
            batches.retrieve_with_provider,
        )

    @cached_property
    def cancel(self) -> CancelResourceWithStreamingResponse:
        return CancelResourceWithStreamingResponse(self._batches.cancel)


class AsyncBatchesResourceWithStreamingResponse:
    def __init__(self, batches: AsyncBatchesResource) -> None:
        self._batches = batches

        self.create = async_to_streamed_response_wrapper(
            batches.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            batches.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            batches.list,
        )
        self.cancel_with_provider = async_to_streamed_response_wrapper(
            batches.cancel_with_provider,
        )
        self.create_with_provider = async_to_streamed_response_wrapper(
            batches.create_with_provider,
        )
        self.list_with_provider = async_to_streamed_response_wrapper(
            batches.list_with_provider,
        )
        self.retrieve_with_provider = async_to_streamed_response_wrapper(
            batches.retrieve_with_provider,
        )

    @cached_property
    def cancel(self) -> AsyncCancelResourceWithStreamingResponse:
        return AsyncCancelResourceWithStreamingResponse(self._batches.cancel)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["InfoListParams"]


class InfoListParams(TypedDict, total=False):
    litellm_model_id: Optional[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Optional
from typing_extensions import TypedDict

__all__ = ["OrganizationUpdateParams"]


class OrganizationUpdateParams(TypedDict, total=False):
    budget_id: Optional[str]

    metadata: Optional[object]

    models: Optional[List[str]]

    organization_alias: Optional[str]

    organization_id: Optional[str]

    spend: Optional[float]

    updated_by: Optional[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from hanzoai import Hanzo, AsyncHanzo
from tests.utils import assert_matches_type
from hanzoai.types import (
    OrganizationListResponse,
    OrganizationCreateResponse,
    OrganizationDeleteResponse,
    OrganizationMembershipTable,
    OrganizationTableWithMembers,
    OrganizationAddMemberResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestOrganization:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create(self, client: Hanzo) -> None:
        organization = client.organization.create(
            organization_alias="organization_alias",
        )
        assert_matches_type(OrganizationCreateResponse, organization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create_with_all_params(self, client: Hanzo) -> None:
        organization = client.organization.create(
            organization_alias="organization_alias",
            budget_duration="budget_duration",
            budget_id="budget_id",
            max_budget=0,
            max_parallel_requests=0,
            metadata={},
            model_max_budget={},
            models=[{}],
            organization_id="organization_id",
            rpm_limit=0,
            soft_budget=0,
            tpm_limit=0,
        )
        assert_matches_type(OrganizationCreateResponse, organization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create(self, client: Hanzo) -> None:
        response = client.organization.with_raw_response.create(
            organization_alias="organization_alias",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        organization = response.parse()
        assert_matches_type(OrganizationCreateResponse, organization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create(self, client: Hanzo) -> None:
        with client.organization.with_streaming_response.create(
            organization_alias="organization_alias",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            organization = response.parse()
            assert_matches_type(OrganizationCreateResponse, organization, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_update(self, client: Hanzo) -> None:
        organization = client.organization.update()
        assert_matches_type(OrganizationTableWithMembers, organization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_update_with_all_params(self, client: Hanzo) -> None:
        organization = client.organization.update(
            budget_id="budget_id",
            metadata={},
            models=["string"],
            organization_alias="organization_alias",
            organization_id="organization_id",
            spend=0,
            updated_by="updated_by",
        )
        assert_matches_type(OrganizationTableWithMembers, organization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_update(self, client: Hanzo) -> None:
        response = client.organization.with_raw_response.update()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        organization = response.parse()
        assert_matches_type(OrganizationTableWithMembers, organization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_update(self, client: Hanzo) -> None:
        with client.organization.with_streaming_response.update() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            organization = response.parse()
            assert_matches_type(OrganizationTableWithMembers, organization, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_list(self, client: Hanzo) -> None:
        organization = client.organization.list()
        assert_matches_type(OrganizationListResponse, organization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_list(self, client: Hanzo) -> None:
        response = client.organization.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        organization = response.parse()
        assert_matches_type(OrganizationListResponse, organization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_list(self, client: Hanzo) -> None:
        with client.organization.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            organization = response.parse()
            assert_matches_type(OrganizationListResponse, organization, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_delete(self, client: Hanzo) -> None:
        organization = client.organization.delete(
            organization_ids=["string"],
        )
        assert_matches_type(OrganizationDeleteResponse, organization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_delete(self, client: Hanzo) -> None:
        response = client.organization.with_raw_response.delete(
            organization_ids=["string"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        organization = response.parse()
        assert_matches_type(OrganizationDeleteResponse, organization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_delete(self, client: Hanzo) -> None:
        with client.organization.with_streaming_response.delete(
            organization_ids=["string"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            organization = response.parse()
            assert_matches_type(OrganizationDeleteResponse, organization, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_add_member(self, client: Hanzo) -> None:
        organization = client.organization.add_member(
            member=[{"role": "org_admin"}],
            organization_id="organization_id",
        )
        assert_matches_type(OrganizationAddMemberResponse, organization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_add_member_with_all_params(self, client: Hanzo) -> None:
        organization = client.organization.add_member(
            member=[
                {
                    "role": "org_admin",
                    "user_email": "user_email",
                    "user_id": "user_id",
                }
            ],
            organization_id="organization_id",
            max_budget_in_organization=0,
        )
        assert_matches_type(OrganizationAddMemberResponse, organization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_add_member(self, client: Hanzo) -> None:
        response = client.organization.with_raw_response.add_member(
            member=[{"role": "org_admin"}],
            organization_id="organization_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        organization = response.parse()
        assert_matches_type(OrganizationAddMemberResponse, organization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_add_member(self, client: Hanzo) -> None:
        with client.organization.with_streaming_response.add_member(
            member=[{"role": "org_admin"}],
            organization_id="organization_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            organization = response.parse()
            assert_matches_type(OrganizationAddMemberResponse, organization, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_delete_member(self, client: Hanzo) -> None:
        organization = client.organization.delete_member(
            organization_id="organization_id",
        )
        assert_matches_type(object, organization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_delete_member_with_all_params(self, client: Hanzo) -> None:
        organization = client.organization.delete_member(
            organization_id="organization_id",
            user_email="user_email",
            user_id="user_id",
        )
        assert_matches_type(object, organization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_delete_member(self, client: Hanzo) -> None:
        response = client.organization.with_raw_response.delete_member(
            organization_id="organization_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        organization = response.parse()
        assert_matches_type(object, organization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_delete_member(self, client: Hanzo) -> None:
        with client.organization.with_streaming_response.delete_member(
            organization_id="organization_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            organization = response.parse()
            assert_matches_type(object, organization, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_update_member(self, client: Hanzo) -> None:
        organization = client.organization.update_member(
            organization_id="organization_id",
        )
        assert_matches_type(OrganizationMembershipTable, organization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_update_member_with_all_params(self, client: Hanzo) -> None:
        organization = client.organization.update_member(
            organization_id="organization_id",
            max_budget_in_organization=0,
            role="proxy_admin",
            user_email="user_email",
            user_id="user_id",
        )
        assert_matches_type(OrganizationMembershipTable, organization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_update_member(self, client: Hanzo) -> None:
        response = client.organization.with_raw_response.update_member(
            organization_id="organization_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        organization = response.parse()
        assert_matches_type(OrganizationMembershipTable, organization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_update_member(self, client: Hanzo) -> None:
        with client.organization.with_streaming_response.update_member(
            organization_id="organization_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            organization = response.parse()
            assert_matches_type(OrganizationMembershipTable, organization, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncOrganization:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create(self, async_client: AsyncHanzo) -> None:
        organization = await async_client.organization.create(
            organization_alias="organization_alias",
        )
        assert_matches_type(OrganizationCreateResponse, organization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncHanzo) -> None:
        organization = await async_client.organization.create(
            organization_alias="organization_alias",
            budget_duration="budget_duration",
            budget_id="budget_id",
            max_budget=0,
            max_parallel_requests=0,
            metadata={},
            model_max_budget={},
            models=[{}],
            organization_id="organization_id",
            rpm_limit=0,
            soft_budget=0,
            tpm_limit=0,
        )
        assert_matches_type(OrganizationCreateResponse, organization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncHanzo) -> None:
        response = await async_client.organization.with_raw_response.create(
            organization_alias="organization_alias",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        organization = await response.parse()
        assert_matches_type(OrganizationCreateResponse, organization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncHanzo) -> None:
        async with async_client.organization.with_streaming_response.create(
            organization_alias="organization_alias",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            organization = await response.parse()
            assert_matches_type(OrganizationCreateResponse, organization, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_update(self, async_client: AsyncHanzo) -> None:
        organization = await async_client.organization.update()
        assert_matches_type(OrganizationTableWithMembers, organization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncHanzo) -> None:
        organization = await async_client.organization.update(
            budget_id="budget_id",
            metadata={},
            models=["string"],
            organization_alias="organization_alias",
            organization_id="organization_id",
            spend=0,
            updated_by="updated_by",
        )
        assert_matches_type(OrganizationTableWithMembers, organization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncHanzo) -> None:
        response = await async_client.organization.with_raw_response.update()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        organization = await response.parse()
        assert_matches_type(OrganizationTableWithMembers, organization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncHanzo) -> None:
        async with async_client.organization.with_streaming_response.update() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            organization = await response.parse()
            assert_matches_type(OrganizationTableWithMembers, organization, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_list(self, async_client: AsyncHanzo) -> None:
        organization = await async_client.organization.list()
        assert_matches_type(OrganizationListResponse, organization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncHanzo) -> None:
        response = await async_client.organization.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        organization = await response.parse()
        assert_matches_type(OrganizationListResponse, organization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncHanzo) -> None:
        async with async_client.organization.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            organization = await response.parse()
            assert_matches_type(OrganizationListResponse, organization, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_delete(self, async_client: AsyncHanzo) -> None:
        organization = await async_client.organization.delete(
            organization_ids=["string"],
        )
        assert_matches_type(OrganizationDeleteResponse, organization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncHanzo) -> None:
        response = await async_client.organization.with_raw_response.delete(
            organization_ids=["string"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        organization = await response.parse()
        assert_matches_type(OrganizationDeleteResponse, organization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncHanzo) -> None:
        async with async_client.organization.with_streaming_response.delete(
            organization_ids=["string"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            organization = await response.parse()
            assert_matches_type(OrganizationDeleteResponse, organization, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_add_member(self, async_client: AsyncHanzo) -> None:
        organization = await async_client.organization.add_member(
            member=[{"role": "org_admin"}],
            organization_id="organization_id",
        )
        assert_matches_type(OrganizationAddMemberResponse, organization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_add_member_with_all_params(self, async_client: AsyncHanzo) -> None:
        organization = await async_client.organization.add_member(
            member=[
                {
                    "role": "org_admin",
                    "user_email": "user_email",
                    "user_id": "user_id",
                }
            ],
            organization_id="organization_id",
            max_budget_in_organization=0,
        )
        assert_matches_type(OrganizationAddMemberResponse, organization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_add_member(self, async_client: AsyncHanzo) -> None:
        response = await async_client.organization.with_raw_response.add_member(
            member=[{"role": "org_admin"}],
            organization_id="organization_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        organization = await response.parse()
        assert_matches_type(OrganizationAddMemberResponse, organization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_add_member(self, async_client: AsyncHanzo) -> None:
        async with async_client.organization.with_streaming_response.add_member(
            member=[{"role": "org_admin"}],
            organization_id="organization_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            organization = await response.parse()
            assert_matches_type(OrganizationAddMemberResponse, organization, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_delete_member(self, async_client: AsyncHanzo) -> None:
        organization = await async_client.organization.delete_member(
            organization_id="organization_id",
        )
        assert_matches_type(object, organization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_delete_member_with_all_params(self, async_client: AsyncHanzo) -> None:
        organization = await async_client.organization.delete_member(
            organization_id="organization_id",
            user_email="user_email",
            user_id="user_id",
        )
        assert_matches_type(object, organization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_delete_member(self, async_client: AsyncHanzo) -> None:
        response = await async_client.organization.with_raw_response.delete_member(
            organization_id="organization_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        organization = await response.parse()
        assert_matches_type(object, organization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_delete_member(self, async_client: AsyncHanzo) -> None:
        async with async_client.organization.with_streaming_response.delete_member(
            organization_id="organization_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            organization = await response.parse()
            assert_matches_type(object, organization, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_member(self, async_client: AsyncHanzo) -> None:
        organization = await async_client.organization.update_member(
            organization_id="organization_id",
        )
        assert_matches_type(OrganizationMembershipTable, organization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_member_with_all_params(self, async_client: AsyncHanzo) -> None:
        organization = await async_client.organization.update_member(
            organization_id="organization_id",
            max_budget_in_organization=0,
            role="proxy_admin",
            user_email="user_email",
            user_id="user_id",
        )
        assert_matches_type(OrganizationMembershipTable, organization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_update_member(self, async_client: AsyncHanzo) -> None:
        response = await async_client.organization.with_raw_response.update_member(
            organization_id="organization_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        organization = await response.parse()
        assert_matches_type(OrganizationMembershipTable, organization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_update_member(self, async_client: AsyncHanzo) -> None:
        async with async_client.organization.with_streaming_response.update_member(
            organization_id="organization_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            organization = await response.parse()
            assert_matches_type(OrganizationMembershipTable, organization, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    strip_not_given,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...types.team import callback_add_params
from ..._base_client import make_request_options

__all__ = ["CallbackResource", "AsyncCallbackResource"]


class CallbackResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CallbackResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return CallbackResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CallbackResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return CallbackResourceWithStreamingResponse(self)

    def retrieve(
        self,
        team_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Get the success/failure callbacks and variables for a team

        Parameters:

        - team_id (str, required): The unique identifier for the team

        Example curl:

        ```
        curl -X GET 'http://localhost:4000/team/dbe2f686-a686-4896-864a-4c3924458709/callback'         -H 'Authorization: Bearer sk-1234'
        ```

        This will return the callback settings for the team with id
        dbe2f686-a686-4896-864a-4c3924458709

        Returns { "status": "success", "data": { "team_id": team_id,
        "success_callbacks": team_callback_settings_obj.success_callback,
        "failure_callbacks": team_callback_settings_obj.failure_callback,
        "callback_vars": team_callback_settings_obj.callback_vars, }, }

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not team_id:
            raise ValueError(f"Expected a non-empty value for `team_id` but received {team_id!r}")
        return self._get(
            f"/team/{team_id}/callback",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def add(
        self,
        team_id: str,
        *,
        callback_name: str,
        callback_vars: Dict[str, str],
        callback_type: Optional[Literal["success", "failure", "success_and_failure"]] | NotGiven = NOT_GIVEN,
        litellm_changed_by: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Add a success/failure callback to a team

        Use this if if you want different teams to have different success/failure
        callbacks

        Parameters:

        - callback_name (Literal["langfuse", "langsmith", "gcs"], required): The name of
          the callback to add
        - callback_type (Literal["success", "failure", "success_and_failure"],
          required): The type of callback to add. One of:
          - "success": Callback for successful LLM calls
          - "failure": Callback for failed LLM calls
          - "success_and_failure": Callback for both successful and failed LLM calls
        - callback_vars (StandardCallbackDynamicParams, required): A dictionary of
          variables to pass to the callback
          - langfuse_public_key: The public key for the Langfuse callback
          - langfuse_secret_key: The secret key for the Langfuse callback
          - langfuse_secret: The secret for the Langfuse callback
          - langfuse_host: The host for the Langfuse callback
          - gcs_bucket_name: The name of the GCS bucket
          - gcs_path_service_account: The path to the GCS service account
          - langsmith_api_key: The API key for the Langsmith callback
          - langsmith_project: The project for the Langsmith callback
          - langsmith_base_url: The base URL for the Langsmith callback

        Example curl:

        ```
        curl -X POST 'http:/localhost:4000/team/dbe2f686-a686-4896-864a-4c3924458709/callback'         -H 'Content-Type: application/json'         -H 'Authorization: Bearer sk-1234'         -d '{
            "callback_name": "langfuse",
            "callback_type": "success",
            "callback_vars": {"langfuse_public_key": "pk-lf-xxxx1", "langfuse_secret_key": "sk-xxxxx"}

        }'
        ```

        This means for the team where team_id = dbe2f686-a686-4896-864a-4c3924458709,
        all LLM calls will be logged to langfuse using the public key pk-lf-xxxx1 and
        the secret key sk-xxxxx

        Args:
          litellm_changed_by: The litellm-changed-by header enables tracking of actions performed by
              authorized users on behalf of other users, providing an audit trail for
              accountability

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not team_id:
            raise ValueError(f"Expected a non-empty value for `team_id` but received {team_id!r}")
        extra_headers = {**strip_not_given({"litellm-changed-by": litellm_changed_by}), **(extra_headers or {})}
        return self._post(
            f"/team/{team_id}/callback",
            body=maybe_transform(
                {
                    "callback_name": callback_name,
                    "callback_vars": callback_vars,
                    "callback_type": callback_type,
                },
                callback_add_params.CallbackAddParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AsyncCallbackResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCallbackResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncCallbackResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCallbackResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return AsyncCallbackResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        team_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Get the success/failure callbacks and variables for a team

        Parameters:

        - team_id (str, required): The unique identifier for the team

        Example curl:

        ```
        curl -X GET 'http://localhost:4000/team/dbe2f686-a686-4896-864a-4c3924458709/callback'         -H 'Authorization: Bearer sk-1234'
        ```

        This will return the callback settings for the team with id
        dbe2f686-a686-4896-864a-4c3924458709

        Returns { "status": "success", "data": { "team_id": team_id,
        "success_callbacks": team_callback_settings_obj.success_callback,
        "failure_callbacks": team_callback_settings_obj.failure_callback,
        "callback_vars": team_callback_settings_obj.callback_vars, }, }

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not team_id:
            raise ValueError(f"Expected a non-empty value for `team_id` but received {team_id!r}")
        return await self._get(
            f"/team/{team_id}/callback",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def add(
        self,
        team_id: str,
        *,
        callback_name: str,
        callback_vars: Dict[str, str],
        callback_type: Optional[Literal["success", "failure", "success_and_failure"]] | NotGiven = NOT_GIVEN,
        litellm_changed_by: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Add a success/failure callback to a team

        Use this if if you want different teams to have different success/failure
        callbacks

        Parameters:

        - callback_name (Literal["langfuse", "langsmith", "gcs"], required): The name of
          the callback to add
        - callback_type (Literal["success", "failure", "success_and_failure"],
          required): The type of callback to add. One of:
          - "success": Callback for successful LLM calls
          - "failure": Callback for failed LLM calls
          - "success_and_failure": Callback for both successful and failed LLM calls
        - callback_vars (StandardCallbackDynamicParams, required): A dictionary of
          variables to pass to the callback
          - langfuse_public_key: The public key for the Langfuse callback
          - langfuse_secret_key: The secret key for the Langfuse callback
          - langfuse_secret: The secret for the Langfuse callback
          - langfuse_host: The host for the Langfuse callback
          - gcs_bucket_name: The name of the GCS bucket
          - gcs_path_service_account: The path to the GCS service account
          - langsmith_api_key: The API key for the Langsmith callback
          - langsmith_project: The project for the Langsmith callback
          - langsmith_base_url: The base URL for the Langsmith callback

        Example curl:

        ```
        curl -X POST 'http:/localhost:4000/team/dbe2f686-a686-4896-864a-4c3924458709/callback'         -H 'Content-Type: application/json'         -H 'Authorization: Bearer sk-1234'         -d '{
            "callback_name": "langfuse",
            "callback_type": "success",
            "callback_vars": {"langfuse_public_key": "pk-lf-xxxx1", "langfuse_secret_key": "sk-xxxxx"}

        }'
        ```

        This means for the team where team_id = dbe2f686-a686-4896-864a-4c3924458709,
        all LLM calls will be logged to langfuse using the public key pk-lf-xxxx1 and
        the secret key sk-xxxxx

        Args:
          litellm_changed_by: The litellm-changed-by header enables tracking of actions performed by
              authorized users on behalf of other users, providing an audit trail for
              accountability

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not team_id:
            raise ValueError(f"Expected a non-empty value for `team_id` but received {team_id!r}")
        extra_headers = {**strip_not_given({"litellm-changed-by": litellm_changed_by}), **(extra_headers or {})}
        return await self._post(
            f"/team/{team_id}/callback",
            body=await async_maybe_transform(
                {
                    "callback_name": callback_name,
                    "callback_vars": callback_vars,
                    "callback_type": callback_type,
                },
                callback_add_params.CallbackAddParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class CallbackResourceWithRawResponse:
    def __init__(self, callback: CallbackResource) -> None:
        self._callback = callback

        self.retrieve = to_raw_response_wrapper(
            callback.retrieve,
        )
        self.add = to_raw_response_wrapper(
            callback.add,
        )


class AsyncCallbackResourceWithRawResponse:
    def __init__(self, callback: AsyncCallbackResource) -> None:
        self._callback = callback

        self.retrieve = async_to_raw_response_wrapper(
            callback.retrieve,
        )
        self.add = async_to_raw_response_wrapper(
            callback.add,
        )


class CallbackResourceWithStreamingResponse:
    def __init__(self, callback: CallbackResource) -> None:
        self._callback = callback

        self.retrieve = to_streamed_response_wrapper(
            callback.retrieve,
        )
        self.add = to_streamed_response_wrapper(
            callback.add,
        )


class AsyncCallbackResourceWithStreamingResponse:
    def __init__(self, callback: AsyncCallbackResource) -> None:
        self._callback = callback

        self.retrieve = async_to_streamed_response_wrapper(
            callback.retrieve,
        )
        self.add = async_to_streamed_response_wrapper(
            callback.add,
        )
